@@ -24,8 +24,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
-int resSubDevFunc();
-
 /*
  *--------------------------------------------------------------------------
  *
@@ -284,19 +282,28 @@ ResEachTile(tile, startpoint)
     for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp=RT(tp))
     {
 	t2 = TiGetRightType(tp);
-	devptr = ExtCurStyle->exts_device[t2];
-	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2) &&
-	      TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t1))
-        /* found device */
+	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2))
 	{
-	    xj = LEFT(tile);
-	    yj = (TOP(tp) + BOTTOM(tp)) >> 1;
-	    ResNewSDDevice(tile, tp, xj, yj, RIGHTEDGE, &ResNodeQueue);
+	    for (devptr = ExtCurStyle->exts_device[t2]; devptr;
+			devptr = devptr->exts_next)
+	    {
+		for (i = 0; i < devptr->exts_deviceSDCount; i++)
+		{
+		    if (TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t1))
+		    {
+			/* found device */
+			xj = LEFT(tile);
+			yj = (TOP(tp) + BOTTOM(tp)) >> 1;
+			ResNewSDDevice(tile, tp, xj, yj, RIGHTEDGE, &ResNodeQueue);
+			break;
+		    }
+		}
+		if (i < devptr->exts_deviceSDCount) break;
+	    }
 	}
 	if TTMaskHasType(&(ExtCurStyle->exts_nodeConn[t1]), t2)
-	/* tile is junction  */
 	{
-	    /*junction rn_loc */
+	    /* tile is junction */
 	    xj = LEFT(tile);
 	    yj = (MAX(BOTTOM(tile), BOTTOM(tp)) + MIN(TOP(tile), TOP(tp))) >> 1;
 	    (void) ResProcessJunction(tile, tp, xj, yj, &ResNodeQueue);
@@ -306,20 +313,29 @@ ResEachTile(tile, startpoint)
     /* right */
     for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp=LB(tp))
     {
-      	t2 = TiGetLeftType(tp);
-	devptr = ExtCurStyle->exts_device[t2];
-	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2) &&
-	      TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t1))
-        /* found device */
+	t2 = TiGetLeftType(tp);
+	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2))
 	{
-	    xj = RIGHT(tile);
-	    yj = (TOP(tp)+BOTTOM(tp))>>1;
-	    ResNewSDDevice(tile, tp, xj, yj, LEFTEDGE, &ResNodeQueue);
+	    for (devptr = ExtCurStyle->exts_device[t2]; devptr;
+			devptr = devptr->exts_next)
+	    {
+		for (i = 0; i < devptr->exts_deviceSDCount; i++)
+		{
+		    if (TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t1))
+		    {
+			/* found device */
+			xj = RIGHT(tile);
+			yj = (TOP(tp) + BOTTOM(tp)) >> 1;
+			ResNewSDDevice(tile, tp, xj, yj, LEFTEDGE, &ResNodeQueue);
+			break;
+		    }
+		}
+		if (i < devptr->exts_deviceSDCount) break;
+	    }
 	}
 	if TTMaskHasType(&ExtCurStyle->exts_nodeConn[t1], t2)
-	/* tile is junction  */
 	{
-	    /*junction rn_loc */
+	    /* tile is junction */
 	    xj = RIGHT(tile);
 	    yj = (MAX(BOTTOM(tile),BOTTOM(tp)) + MIN(TOP(tile), TOP(tp))) >> 1;
 	    (void)ResProcessJunction(tile, tp, xj, yj, &ResNodeQueue);
@@ -327,21 +343,31 @@ ResEachTile(tile, startpoint)
     }
 
     /* top */
-    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp=BL(tp))
+    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
     {
-      	t2 = TiGetBottomType(tp);
-	devptr = ExtCurStyle->exts_device[t2];
-	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2) &&
-	      TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t1))
-        /* found device */
+	t2 = TiGetBottomType(tp);
+	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2))
 	{
-	    yj = TOP(tile);
-	    xj = (LEFT(tp)+RIGHT(tp))>>1;
-	    ResNewSDDevice(tile, tp, xj, yj, BOTTOMEDGE, &ResNodeQueue);
+	    for (devptr = ExtCurStyle->exts_device[t2]; devptr;
+			devptr = devptr->exts_next)
+	    {
+		for (i = 0; i < devptr->exts_deviceSDCount; i++)
+		{
+		    if(TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t1))
+		    {
+			/* found device */
+			yj = TOP(tile);
+			xj = (LEFT(tp) + RIGHT(tp)) >> 1;
+			ResNewSDDevice(tile, tp, xj, yj, BOTTOMEDGE, &ResNodeQueue);
+			break;
+		    }
+		}
+		if (i < devptr->exts_deviceSDCount) break;
+	    }
 	}
-	if TTMaskHasType(&ExtCurStyle->exts_nodeConn[t1],t2)
-	/* tile is junction  */
+	if TTMaskHasType(&ExtCurStyle->exts_nodeConn[t1], t2)
 	{
+	    /* tile is junction */
 	    yj = TOP(tile);
 	    xj = (MAX(LEFT(tile),LEFT(tp)) + MIN(RIGHT(tile),RIGHT(tp))) >> 1;
 	    ResProcessJunction(tile, tp, xj, yj, &ResNodeQueue);
@@ -351,19 +377,29 @@ ResEachTile(tile, startpoint)
     /* bottom */
     for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp=TR(tp))
     {
-      	t2 = TiGetTopType(tp);
-	devptr = ExtCurStyle->exts_device[t2];
-	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2) &&
-	      TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t1))
-        /* found device */
+	t2 = TiGetTopType(tp);
+	if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask), t2))
 	{
-	    yj = BOTTOM(tile);
-	    xj = (LEFT(tp) + RIGHT(tp)) >> 1;
-	    ResNewSDDevice(tile, tp, xj, yj, TOPEDGE, &ResNodeQueue);
+	    for (devptr = ExtCurStyle->exts_device[t2]; devptr;
+			devptr = devptr->exts_next)
+	    {
+		for (i = 0; i < devptr->exts_deviceSDCount; i++)
+		{
+		    if (TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t1))
+		    {
+			/* found device */
+			yj = BOTTOM(tile);
+			xj = (LEFT(tp) + RIGHT(tp)) >> 1;
+			ResNewSDDevice(tile, tp, xj, yj, TOPEDGE, &ResNodeQueue);
+			break;
+		    }
+		}
+		if (i < devptr->exts_deviceSDCount) break;
+	    }
 	}
 	if TTMaskHasType(&(ExtCurStyle->exts_nodeConn[t1]), t2)
-	/* tile is junction  */
 	{
+	    /* tile is junction */
 	    yj = BOTTOM(tile);
 	    xj = (MAX(LEFT(tile),LEFT(tp)) + MIN(RIGHT(tile),RIGHT(tp))) >> 1;
 	    ResProcessJunction(tile, tp, xj, yj, &ResNodeQueue);
@@ -440,45 +476,3 @@ ResEachTile(tile, startpoint)
 
     return(merged);
 }
-
-/*
- *-------------------------------------------------------------------------
- *
- * resSubDevFunc -- called when DBSrPaintArea finds a device within
- *	a substrate area.
- *
- * Results: always returns 0 to keep search going.
- *
- * Side Effects: allocates substrate node.
- *
- *-------------------------------------------------------------------------
- */
-
-int
-resSubDevFunc(tile, tp)
-    Tile	*tile, *tp;
-{
-    tileJunk	*junk = (tileJunk *)(tile->ti_client);
-    resNode	*resptr;
-    tElement	*tcell;
-    int		x, y;
-
-    if (junk->deviceList->rd_fet_subs == NULL)
-    {
-        resptr = (resNode *) mallocMagic((unsigned)(sizeof(resNode)));
-	junk->deviceList->rd_fet_subs = resptr;
-	junk->tj_status |= RES_TILE_DEV;
-        tcell = (tElement *) mallocMagic((unsigned)(sizeof(tElement)));
-	tcell->te_thist = junk->deviceList;
-	tcell->te_nextt = NULL;
-	x = (LEFT(tile) + RIGHT(tile)) >> 1;
-	y = (TOP(tile) + BOTTOM(tile)) >> 1;
-
-	InitializeNode(resptr, x, y, RES_NODE_JUNCTION);
-	resptr->rn_te = tcell;
-	ResAddToQueue(resptr, &ResNodeQueue);
-
-	NEWBREAK(resptr, tp, x, y, NULL);
-    }
-    return 0;
-}
